




export class SpineAnimation {
    private __node: cc.Node | null;
    private __skeleton: sp.Skeleton | null;
    private __url: { url: string, bundle: string } | null = null;

    constructor(node: string | cc.Node, cacheMode: number = 1, enableBatch: boolean = true) {
        if (node instanceof cc.Node) {
            this.__node = node;
        } else {
            this.__node = new cc.Node(node);
        }
        this.__skeleton = this.__node.getComponent(sp.Skeleton);
        if (this.__skeleton == null) {
            this.__skeleton = this.__node.addComponent(sp.Skeleton);
        }
        if (cacheMode == 0) {
            this.__skeleton.setAnimationCacheMode(sp.Skeleton.AnimationCacheMode.REALTIME);
        } else if (cacheMode == 1) {
            this.__skeleton.setAnimationCacheMode(sp.Skeleton.AnimationCacheMode.SHARED_CACHE);
        } else {
            this.__skeleton.setAnimationCacheMode(sp.Skeleton.AnimationCacheMode.PRIVATE_CACHE);
        }
        // this.__skeleton.premultipliedAlpha = false;
        this.__skeleton.enableBatch = false;
    }

    set url(value: { url: string, bundle: string } | null) {
        this.__url = value;
        this.__urlChanged();
    }

    get url(): { url: string, bundle: string } | null {
        return this.__url!;
    }

    private __urlChanged(): void {
        let bundle: cc.AssetManager.Bundle | null = cc.assetManager.getBundle(this.url!.bundle);
        if (bundle) {
            let res: sp.SkeletonData | null = bundle.get(this.url!.url);
            if (res) {
                this.__assetLoaded(null, res);
            } else {
                bundle.load(this.url!.url, sp.SkeletonData, this.__assetLoaded.bind(this));
            }
        } else {
            cc.assetManager.loadBundle(this.url!.bundle, null, this.__bundleLoaded.bind(this));
        }
    }

    private __bundleLoaded(err: Error | null, bundle: cc.AssetManager.Bundle): void {
        if (err) {
            throw new Error(err.message);
        }
        let res: sp.SkeletonData | null = bundle.get(this.url!.url);
        if (res) {
            this.__assetLoaded(null, res);
        } else {
            bundle.load(this.url!.url, sp.SkeletonData, this.__assetLoaded.bind(this));
        }
    }

    private __assetLoaded(err: Error | null, skData: sp.SkeletonData): void {
        if (err) {
            throw new Error(err.message);
        }
        this.__skeleton!.skeletonData = skData;
        this.__skeleton!.setAnimation(0, this.randomAnimation, true);
    }


    playRandomAnimation(): void {
        this.__skeleton!.setAnimation(0, this.randomAnimation, true);
    }

    get boneCount(): number {
        let skData: any = this.__skeleton!.skeletonData;
        if (!skData) {
            return 0;
        }
        return skData["_skeletonCache"]["bones"].length;
    }

    get randomAnimation(): string {
        let skData: any = this.__skeleton!.skeletonData;
        let list = skData["_skeletonCache"]["animations"]
        let index: number = Math.ceil(Math.random() * (list.length - 1));
        let ani = list[index];
        return ani.name;
    }

    get node(): cc.Node {
        return this.__node!;
    }

    get skeleton(): sp.Skeleton {
        return this.__skeleton!;
    }

    destroy(): void {
        this.__skeleton!.destroy();
        this.__node!.destroy();
        this.__node = null;
    }
}